% % create file nlcon.m for nonlinear constraints
% function [c,ceq] = nlcon(x)
%   c = 25.0 - x(1)*x(2)*x(3)*x(4);
%   ceq = sum(x.^2) - 40;



objective = @(x) x(1)*x(4)*(x(1)+x(2)+x(3))+x(3);

% initial guess
x0 = [1,5,5,1];

% variable bounds
lb = 1.0 * ones(4);
ub = 5.0 * ones(4);

% show initial objective
disp(['Initial Objective: ' num2str(objective(x0))])

% linear constraints
A = [];
b = [];
Aeq = [];
beq = [];

% nonlinear constraints
nonlincon = @nlcon;

% optimize with fmincon
%[X,FVAL,EXITFLAG,OUTPUT,LAMBDA,GRAD,HESSIAN]
% = fmincon(FUN,X0,A,B,Aeq,Beq,LB,UB,NONLCON,OPTIONS)
x = fmincon(objective,x0,A,b,Aeq,beq,lb,ub,nonlincon);

% show final objective
disp(['Final Objective: ' num2str(objective(x))])

% print solution
disp('Solution')
disp(['x1 = ' num2str(x(1))])
disp(['x2 = ' num2str(x(2))])
disp(['x3 = ' num2str(x(3))])
disp(['x4 = ' num2str(x(4))])


%% esempio fmincon di matlab

fun = @(x)100*(x(2)-x(1)^2)^2 + (1-x(1))^2;

x0 = [-1,2];
A = [1,2];
b = 1;
x = fmincon(fun,x0,A,b);


%% prova creazione di una funzione integrale

fun_int =@(x) exp(-x.^2).*log(x).^2;

int_1 =@(a,b) integral(fun_int,a,b);

input_1 =0;
input_2 =inf;
risposta =int_1(input_1,input_2);


disp(['risposta = ' num2str(risposta)])


%% prova creazione di una funzione integrale
clear
clc

%inserimento dati (FISSI)
m_v =1645;
delta =1.05;
v_climb =15;
rt =0.315;
f_volv =0.0083;
perc_peso= 0.55;
m_v_1 =m_v*perc_peso;
eta_pt= 0.95; %efficienza di trasmissione assunta 95%
g =9.81;
rho_aria =1.225;
phi_50 =0;
Cd =0.28;
T_em =254;
P_nom= 80; %Potenza Nominale del motore [kW] (rated power)
S_ev =2.19;
v_max =205;
n_mot= 10000; %velocità massima raggiunta dal motore [rpm] (peak velocity)
i_0tr= 1; %valore del riduttore
T_em_maxspeed =76.3944;



%parte fissa del ragionamento, infatti la coppia resistente è sempre la stessa
%calcolo le forze che entrano in gioco per trovare la coppia resistente (cerco velocità di cambio)
F_aerodinamica_50 =0.5*rho_aria*Cd*S_ev*((50/3.6)^2); %forza aerodinamica alla velocità media di 50 km/h
F_attrito_50 =f_volv*m_v_1*g*cosd(phi_50);
F_climbing_50 =m_v_1*g*sind(phi_50);

F_req_50 =F_aerodinamica_50+F_attrito_50+F_climbing_50;
Tw_req_50 =F_req_50*rt; %[Nm] coppia resistente con marcia-1 ingranata

n_mot_rad_s =n_mot*(pi/30); %trasformo n_mot da rpm a rad/s
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% (fine parte dati FISSI)


%inserimento dati (VARIABILI)
marcia.seconda =3.4; %%%(varia in un range)
marcia.prima =8.5; %%%(varia in un range)
T_em_marcia1 =T_em*marcia.prima; %%%(moltiplicazione per un range)
vel_cambio_kmh_pp =20.1637; %%% PROBLEMATICO!

%%%%%%%%% DEVO TORVARE IL PUNTO DI INTERSEZIONE TRA LE DUE CURVE

a_1=0;
b_2 =round(vel_cambio_kmh_pp/3.6);
c_3=b_2;
d_4=27.8;

fun_1 = @(x) (m_v*delta)./(  ((T_em_marcia1*v_climb)./(rt*3.6.*x)) - F_attrito_50 - ( ((Cd*S_ev)/(21.15))*((3.6.*x).^(2)) )  );
fun_2 = @(x) (m_v*delta)./(  ((T_em_maxspeed*marcia.seconda*v_max)./(rt*3.6.*x)) - F_attrito_50 - ( ((Cd*S_ev)/(21.15))*((3.6.*x).^(2)) )  );

int_finale =@(a,b,c,d) integral(fun_1,a,b) +integral(fun_2,c,d) +2;
risposta =int_finale(a_1,b_2,c_3,d_4);


disp(['risposta = ' num2str(risposta)])


