%% inizializzazione valori per debug (presi da: 10-Development of continuously variable transmission and multi-speed dualclutch transmission for pure electric vehicle)

clear
clc
close all

% CLASSE-B
m_v= 1400; %[kg] 
f_volv= 0.013;
phi= 50; %angolo di inclinazione massimo della strada [°]
Cd= 0.28; %coefficiente di resistenza aerodinamica del veicolo
rho_aria= 1.127; %densità dell'aria espressa in [kg/m^3]
S_ev= 2.47; %[m^2]
v_climb= 10; %velocità in salita del veicolo [km/h] (assunta)
%%%(The aerodynamic drag is neglected at the low vehicle speeds)
rt= 0.302; %[m] raggio della ruota 315.95 mm
eta_pt= 0.86; %efficienza di trasmissione (two-speed overall efficiency)
g= 9.81; %accelerazione di gravità [m/s^2]
T_em= 250; % [N*m] coppia nominale erogabile dal motore (dal grafico)
P_nom= 65; %[kW] Potenza Nominale del motore (Motor peak power)
v_max= 180; %velocità massima ipotizzabile raggiungibile dal veicolo [km/h]
n_mot= 6250; %velocità massima raggiunta dal motore [rpm] (dal grafico)

% CLASSE-E
% m_v= 2200; %[kg] 
% f_volv= 0.013;
% phi= 50; %angolo di inclinazione massimo della strada [°]
% Cd= 0.3; %coefficiente di resistenza aerodinamica del veicolo
% rho_aria= 1.127; %densità dell'aria espressa in [kg/m^3]
% S_ev= 2.68; %[m^2]
% v_climb= 10; %velocità in salita del veicolo [km/h] (assunta)
% %%%(The aerodynamic drag is neglected at the low vehicle speeds)
% rt= 0.344; %[m] raggio della ruota 315.95 mm
% eta_pt= 0.86; %efficienza di trasmissione (two-speed overall efficiency)
% g= 9.81; %accelerazione di gravità [m/s^2]
% T_em= 350; % [N*m] coppia nominale erogabile dal motore (dal grafico)
% P_nom= 110; %[kW] Potenza Nominale del motore (Motor peak power)
% v_max= 200; %velocità massima ipotizzabile raggiungibile dal veicolo [km/h]
% n_mot= 9000; %velocità massima raggiunta dal motore [rpm] (dal grafico)


%% creazione curva caratteristica motore

n_trans =(30*P_nom*1000)./(pi*T_em);
x1 =linspace(0,n_trans,10000);
T_funz_Tmax =T_em+x1*0;
x2 =linspace(n_trans,n_mot,10000);
T_funz =(30*P_nom*1000)./(pi*x2);

%grafico momento torcente motore
Grafico =plot(x1,T_funz_Tmax,'b-',x2,T_funz,'b-');

%calcolo il valore della coppia alla massima velocita del motore
T_em_maxspeed =(30*P_nom*1000)/(pi*n_mot);


%% calcolo intervalli di esistenza rapp. trasmissione totali (riduttore + marcia) marcia 1 e 2

%calcolo le forze che entrano in gioco nella marcia-1
F_aerodinamica_1 =0.5*rho_aria*Cd*S_ev*((v_climb/3.6)^2);
F_attrito_1 =f_volv*m_v*g*cosd(phi);
F_climbing_1 =m_v*g*sind(phi);

%calcolo rapporto min marcia 1 (il minimo rapporto che serve per far andare il veicolo in salita)
i_1min =(rt*(F_attrito_1+F_climbing_1+F_aerodinamica_1))/(eta_pt*T_em);

%calcolo le forze che entrano in gioco nella marcia-2
F_aerodinamica_2 =0.5*rho_aria*Cd*S_ev*((v_max*(1000/3600))^2);
F_attrito_2 =f_volv*m_v*g;
%F_climbing qua è pari a zero perche si ipotizza che la massima velocita si raggiunge in piano

%calcolo rapporto max marcia 2 (il massimo rapporto raggiungibile con la velocita ipotizzata)
%i_2max deve essere MINORE di i_2max_maxspeed perchè: (i_2max < i_2max_maxspeed)
%ho raggiunto la max velocità imposta con la max velocità del motore, posso raggiungere la stessa velcoita massima con i_2max PIU PICCOLI diminuendo contestualmente la vel del motore
i_2max_maxspeed =((pi/30)*n_mot*rt)/(v_max*(1000/3600));
%i_2max deve essere MAGGIORE di i_2max_maxtorque perchè: (i_2max > i_2max_maxtorque)
%mi trovo alla velocita max del motore, per raggiungere la Vmax del veicolo imposta devo avere ALMENO questo rapporto di trasmissione
i_2max_maxtorque =(rt*(F_attrito_2+F_aerodinamica_2))/(eta_pt*T_em_maxspeed);

%controllo valore marcia 2
if i_2max_maxspeed > i_2max_maxtorque
    i_2max= i_2max_maxspeed;
    %la velocità massima raggiungibile ipotizzata del veicolo è consona alla taglia del motore preso in considerazione
    %calcolo la velocità limite massima raggiungibile dal veicolo che può sopportare la taglia del motore in esame
    v_max_i2_accettabile =((pi/30)*n_mot*rt)/(i_2max_maxtorque*(1000/3600));
else
    %condizione NON ACCETABILE:
    fprintf ('\nATTENZIONE!!! (1) \n\nPer le condizioni di lavoro ipotizzare (Vmax) il motore scelto ha dei valori di taglia troppo piccoli! \nInfatti ottengo un assurdo sulla condizione di esistenza del rapporto di trasmissione max della marcia-2:\n i2_max < %f & i2_max > %f', i_2max_maxspeed, i_2max_maxtorque);
    %o si cambia la Vmax del veiolo: calcolo la velocità massima raggiungibile dal veicolo con il motore in esame
    v_max_i2_accettabile =((pi/30)*n_mot*rt)/(i_2max_maxtorque*(1000/3600));
    %o si cambia la taglia del motore: tengo costanti le ipotesi di lavoro e calcolo la taglia del motore che mi serve
    %aumento il numero di giri max a 10000 aumentando contestualmnte la potenza nominale
    
    i_2_mt =i_2max_maxtorque;
    i_2_mv =i_2max_maxspeed;
    P_nom_i =P_nom;
    n_mot_i =n_mot;
    while i_2_mv < i_2_mt
          if n_mot_i < 10000
             n_mot_i =n_mot_i+50;    
          end
          P_nom_i =P_nom_i+2;
          T_em_mv_i =(30*P_nom_i*1000)/(pi*n_mot_i); 
          i_2_mv =((pi/30)*n_mot_i*rt)/(v_max*(1000/3600));
          i_2_mt =(rt*(F_attrito_2+F_aerodinamica_2))/(eta_pt*T_em_mv_i);      
    end 
    
    i_2max =i_2_mv;
    P_em_minima =P_nom_i;
    fprintf ('\n\nSe si considera valido il motore in esame, la velocità massima raggiungibile da tale veicolo deve essere pari a: %f [km/h]\n', v_max_i2_accettabile);
    fprintf ('Se si considerano valide le condizioni di lavoro ipotizzare, il motore deve avere una potenza nominale pari a: %f [N*m]\n', P_em_minima);
end

%Calcolo gli estremi secondo Lechner (va da 2,2 a 3,4)
i_1max =i_2max*3.4;
i_2min =i_1min/3.4;

%riassunto dati ottenuti
i_1 =struct('min',i_1min,'max',i_1max);
fprintf ('\nIntervallo in cui trovare il rapporto di trasmissione totale marcia-1 (riduttore + marcia 1) ottimale è:\n');
disp (i_1);
i_2 =struct('min',i_2min,'max',i_2max);
fprintf ('Intervallo in cui trovare il rapporto di trasmissione totale marcia-2 (riduttore + marcia 2) ottimale è:\n');
disp (i_2);

%verifica validità dati ottenuti
if i_1.min > i_1.max || i_2.min > i_2.max
    fprintf ('\nATTENZIONE!!! (2) \nTaglia motore troppo piccola per le condizioni di lavoro individuate in fase di progetto. \n(Ottengo un assurdo negli estremi degli intervalli di esistenza dei rapporti di trasmissione)\n');
    %calcolo il valore di coppia minima che permette di ottenere le condizioni di lavoro ipotizzate.
    %dall espressione di i_2max mi ricavo i_1max. successivamente considero i_1min=i_1max per ottenere 
    %la minima coppia che non mi genera un assurdo negli intervalli di esistenza dei rapporti di trasmissione
    i_1_min_prova =i_1.max;
    T_em_minima =(rt*(F_attrito_1+F_climbing_1+F_aerodinamica_1))/(i_1_min_prova*eta_pt);
    fprintf ('\nPer le condizioni di lavoro ipotizzare il motore scelto deve avere un valore minimo di coppia nominale pari a: %f [N*m]\n', T_em_minima);
else
    if i_1.max > 40
        fprintf ('\nATTENZIONE!!! (3) \nPer le condizioni di lavoro ipotizzare il motore scelto ha dei valori di taglia troppo piccoli. ottengo un i_1.max troppo alto: %f\n', i_1.max);
    else 
        fprintf ('\ncaratteristiche del motore compatibili con le condizioni di lavoro ipotizzate.\n');
    end

end