% Programma ottimo generale, Vel_0_100 + SOC

clear
clc

%inserimento dati (FISSI)
m_v =1645;
perc_peso= 0.55;
m_v_1 =m_v*perc_peso;
delta =1.05;
v_climb =15;
rt =0.315;
f_volv =0.0083;
eta_pt= 0.95; %efficienza di trasmissione assunta 95%
g =9.81;
rho_aria =1.225;
phi_50 =0;
phi_i =0;
phi= 30; %angolo di inclinazione massimo della strada (per le prove che fa nel documento viene considerato nullo)
Cd =0.28;
selez6 =1;
selez2 =2;

T_em =254;
P_nom= 80; %Potenza Nominale del motore [kW] (rated power)
S_ev =2.19;
v_max =205;
n_mot= 10000; %velocità massima raggiunta dal motore [rpm] (peak velocity)
n_mot_rad_s =n_mot*(pi/30); %trasformo n_mot da rpm a rad/s
i_0tr= 1; %valore del riduttore


T_ref =250; %[Nm]
n_ref =10000*(pi/30); %[rad/s]
P_nom_ref =75; %[kw]

kc =0.2; % Perdita nel rame
ki =0.008; % Perdita nel ferro
kw =0.00001; % coefficiente di perdita nell'aria
C =400; % perdita costante nel motore

parte_1 =(T_em*n_mot_rad_s)/(T_ref*n_ref);

SOC =0.9; %valore iniziale della batteria
delta_t =1; %intervallo in cui studio la variazione della batteria [sec]
Ud_k =300; %tensione batteria [V]
acceleraz_max =2.8571; %accelerazione massima raggiunta durante la prova NEDC
C_batt= 70; %capacità batteria espressa in Ampere ora [A h]

%parte fissa del ragionamento, infatti la coppia resistente è sempre la stessa
%calcolo le forze che entrano in gioco per trovare la coppia resistente (cerco velocità di cambio)
F_aerodinamica_50 =0.5*rho_aria*Cd*S_ev*((50/3.6)^2); %forza aerodinamica alla velocità media di 50 km/h
F_attrito_50 =f_volv*m_v_1*g*cosd(phi_50);
F_climbing_50 =m_v_1*g*sind(phi_50);

F_req_50 =F_aerodinamica_50+F_attrito_50+F_climbing_50;
Tw_req_50 =F_req_50*rt; %[Nm] coppia resistente con marcia-1 ingranata

F_attrito_i =f_volv*m_v_1*g*cosd(phi_i);
F_climbing_i =m_v_1*g*sind(phi_i);

%estremi integrazione fissi
a_1=0;
d_4=27.8;

%scelta peso da dare ai due metodi
peso_SOC =0.3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n_trans =(30*P_nom*1000)./(pi*T_em);
x1 =linspace(0,n_trans,10000);
T_funz_Tmax =T_em+x1*0;
x2 =linspace(n_trans,n_mot,10000);
T_funz =(30*P_nom*1000)./(pi*x2);

%grafico momento torcente motore
figure
Grafico_1 =plot(x1,T_funz_Tmax,'b-',x2,T_funz,'b-');
xlabel('Speed/rpm'), ylabel('Torque/N.m');
title('Grafico coppia massima motore') 

%calcolo il valore della coppia alla massima velocita del motore
T_em_maxspeed =(30*P_nom*1000)/(pi*n_mot);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%calcolo le forze che entrano in gioco nella marcia-1
F_aerodinamica_1 =0.5*rho_aria*Cd*S_ev*((v_climb/3.6)^2);
F_attrito_1 =f_volv*m_v_1*g*cosd(phi);
F_climbing_1 =m_v_1*g*sind(phi);

%calcolo rapporto min marcia 1 (il minimo rapporto che serve per far andare il veicolo in salita)
i_1min =(rt*(F_attrito_1+F_climbing_1+F_aerodinamica_1))/(eta_pt*T_em);

%calcolo le forze che entrano in gioco nella marcia-2
F_aerodinamica_2 =0.5*rho_aria*Cd*S_ev*((v_max*(1000/3600))^2);
F_attrito_2 =f_volv*m_v_1*g;
%F_climbing qua è pari a zero perche si ipotizza che la massima velocita si raggiunge in piano

%calcolo rapporto max marcia 2 (il massimo rapporto raggiungibile con la velocita ipotizzata)
%i_2max deve essere MINORE di i_2max_maxspeed perchè: (i_2max < i_2max_maxspeed)
%ho raggiunto la max velocità imposta con la max velocità del motore, posso raggiungere la stessa velcoita massima con i_2max PIU PICCOLI diminuendo contestualmente la vel del motore
i_2max_maxspeed =((pi/30)*n_mot*rt)/(v_max*(1000/3600));
%i_2max deve essere MAGGIORE di i_2max_maxtorque perchè: (i_2max > i_2max_maxtorque)
%mi trovo alla velocita max del motore, per raggiungere la Vmax del veicolo imposta devo avere ALMENO questo rapporto di trasmissione
i_2max_maxtorque =(rt*(F_attrito_2+F_aerodinamica_2))/(eta_pt*T_em_maxspeed);

%controllo valore marcia 2
if i_2max_maxspeed > i_2max_maxtorque
    i_2max= i_2max_maxspeed;
    %la velocità massima raggiungibile ipotizzata del veicolo è consona alla taglia del motore preso in considerazione
    %calcolo la velocità limite massima raggiungibile dal veicolo che può sopportare la taglia del motore in esame
    v_max_i2_accettabile =((pi/30)*n_mot*rt)/(i_2max_maxtorque*(1000/3600));
else
    %condizione NON ACCETABILE:
    fprintf ('\nATTENZIONE!!! (1) \nPer le condizioni di lavoro ipotizzare (Vmax) il motore scelto ha dei valori di taglia troppo piccoli! \nInfatti ottengo un assurdo sulla condizione di esistenza del rapporto di trasmissione max della marcia-2:\n i2_max < %.2f & i2_max > %.2f', i_2max_maxspeed, i_2max_maxtorque);
    %o si cambia la Vmax del veiolo: calcolo la velocità massima raggiungibile dal veicolo con il motore in esame
    v_max_i2_accettabile =((pi/30)*n_mot*rt)/(i_2max_maxtorque*(1000/3600));
    %o si cambia la taglia del motore: tengo costanti le ipotesi di lavoro e calcolo la taglia del motore che mi serve
    %aumento il numero di giri max a 10000 aumentando contestualmnte la potenza nominale
    
    i_2_mt =i_2max_maxtorque;
    i_2_mv =i_2max_maxspeed;
    P_nom_i =P_nom;
    n_mot_i =n_mot;
    while i_2_mv < i_2_mt
          if n_mot_i < 10000
             n_mot_i =n_mot_i+50;    
          end
          P_nom_i =P_nom_i+2;
          T_em_mv_i =(30*P_nom_i*1000)/(pi*n_mot_i); 
          i_2_mv =((pi/30)*n_mot_i*rt)/(v_max*(1000/3600));
          i_2_mt =(rt*(F_attrito_2+F_aerodinamica_2))/(eta_pt*T_em_mv_i);      
    end 
    
    i_2max =i_2_mv;
    P_em_minima =P_nom_i;
    fprintf ('\n\nSe si considera valido il motore in esame, la velocità massima raggiungibile da tale veicolo deve essere pari a: %.2f [km/h]\n', v_max_i2_accettabile);
    fprintf ('Se si considerano valide le condizioni di lavoro ipotizzare, il motore deve avere una potenza nominale pari a: %.2f [N*m]\n', P_em_minima);
end

%Calcolo gli estremi secondo Lechner (va da 2,2 a 3,4)
i_1max =i_2max*3.4;
i_2min =i_1min/3.4;

%riassunto dati ottenuti

if selez2 == 1
    i_1 =struct('min',(ceil(i_1min*10))/10, 'max',(floor(i_1max*10))/10);
    fprintf ('\n\nIntervallo in cui trovare il rapporto di trasmissione marcia-1 ottimale è (tenendo conto che i_0 (riduttore)=%.2f):\n', i_0tr);
    fprintf ('    min: %.3f (%.2f) [con rid: %.2f]\n    max: %.3f (%.2f) [con rid: %.2f]\n\n',i_1min,i_1.min,i_1.min/i_0tr,i_1max,i_1.max,i_1.max/i_0tr);
    
    i_2 =struct('min',(ceil(i_2min*10))/10, 'max',(floor(i_2max*10))/10);
    fprintf ('Intervallo in cui trovare il rapporto di trasmissione marcia-2 ottimale è (tenendo conto che i_0 (riduttore)=%.2f):\n', i_0tr);
    fprintf ('    min: %.3f (%.2f) [con rid: %.2f]\n    max: %.3f (%.2f) [con rid: %.2f]\n\n',i_2min,i_2.min,(i_2.min/i_0tr),i_2max,i_2.max,(i_2.max/i_0tr));
else 
    i_1 =struct('min',(ceil(i_1min*10))/10, 'max',(floor(i_1max*10))/10);
    fprintf ('\n\nIntervallo in cui trovare il rapporto di trasmissione totale marcia-1 (riduttore + marcia 1) ottimale è:\n');
    fprintf ('    min: %.3f (%.2f)\n    max: %.3f (%.2f)\n\n',i_1min,i_1.min,i_1max,i_1.max);
    
    i_2 =struct('min',(ceil(i_2min*10))/10, 'max',(floor(i_2max*10))/10);
    fprintf ('Intervallo in cui trovare il rapporto di trasmissione totale marcia-2 (riduttore + marcia 2) ottimale è:\n');
    fprintf ('    min: %.3f (%.2f)\n    max: %.3f (%.2f)\n\n',i_2min,i_2.min,i_2max,i_2.max);
    
    i_0tr =1;
    
end

%verifica validità dati ottenuti
if i_1.min > i_1.max || i_2.min > i_2.max
    fprintf ('\nATTENZIONE!!! (2) \nTaglia motore troppo piccola per le condizioni di lavoro individuate in fase di progetto. \n(Ottengo un assurdo negli estremi degli intervalli di esistenza dei rapporti di trasmissione)\n');
    %calcolo il valore di coppia minima che permette di ottenere le condizioni di lavoro ipotizzate.
    %dall espressione di i_2max mi ricavo i_1max. successivamente considero i_1min=i_1max per ottenere 
    %la minima coppia che non mi genera un assurdo negli intervalli di esistenza dei rapporti di trasmissione
    i_1_min_prova =i_1.max;
    T_em_minima =(rt*(F_attrito_1+F_climbing_1+F_aerodinamica_1))/(i_1_min_prova*eta_pt);
    fprintf ('\nPer le condizioni di lavoro ipotizzare il motore scelto deve avere un valore minimo di coppia nominale pari a: %f [N*m]\n(La coppia nominale ipotizzata è di: %.2f[N*m])\n', T_em_minima, T_em);
else
    if i_1.max > 40
        fprintf ('\nATTENZIONE!!! (3) \nPer le condizioni di lavoro ipotizzare il motore scelto ha dei valori di taglia troppo piccoli. ottengo un i_1.max troppo alto: %.2f\n', i_1.max);
    else 
        fprintf ('\ncaratteristiche del motore compatibili con le condizioni di lavoro ipotizzate.\n');
    end

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parte SOC

vel_istanti =zeros(1,1179);
for i=1:1179

    vel_istanti(1,i) =vel_veicolo(i);

end
            
                       
F_accelerazione =m_v*delta*((vel_istanti(1,2:end)-vel_istanti(1,1:end-1))/3.6);
                       
Tw_req_k_vett_pm =  ((0.5*rho_aria*Cd*S_ev*((vel_istanti(1,2:end)./3.6).^2))+F_attrito_i+F_climbing_i+F_accelerazione)*rt; 
                          
Tw_req_k_vett = (Tw_req_k_vett_pm >=0).*Tw_req_k_vett_pm + ((Tw_req_k_vett_pm)<0).*(Tw_req_k_vett_pm.*(-1));



P_em_i_1_vett =@(x) (Tw_req_k_vett./(x(1).*i_0tr*eta_pt)).*(((30*x(1)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30));

P_loss_1_vett =@(x)  parte_1.*(((kc*(((Tw_req_k_vett./(T_em*x(1)))*T_em).^2).*(T_ref/T_em)^2))+...
                     (ki.*(((30*x(1)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).*(n_ref/n_mot_rad_s))+...
                     (kw.*((((30*x(1)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).^3).*(n_ref/n_mot_rad_s)^3)+C);
 
fun_eta_1_vett =@(x) ((Tw_req_k_vett./(x(1).*i_0tr*eta_pt)).*(((30*x(1)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)))./...
                     (((Tw_req_k_vett./(x(1).*i_0tr*eta_pt)).*(((30*x(1)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)))+...
                     (parte_1.*(((kc*(((Tw_req_k_vett./(T_em*x(1)))*T_em).^2).*(T_ref/T_em)^2))+...
                     (ki.*(((30*x(1)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).*(n_ref/n_mot_rad_s))+...
                     (kw.*((((30*x(1)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).^3).*(n_ref/n_mot_rad_s)^3)+C)));
 

P_em_i_2_vett =@(x) (Tw_req_k_vett./(x(2).*i_0tr*eta_pt)).*(((30*x(2)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30));

P_loss_2_vett =@(x)  parte_1.*(((kc*(((Tw_req_k_vett./(T_em*x(2)))*T_em).^2).*(T_ref/T_em)^2))+...
                     (ki.*(((30*x(2)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).*(n_ref/n_mot_rad_s))+...
                     (kw.*((((30*x(2)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).^3).*(n_ref/n_mot_rad_s)^3)+C);

fun_eta_2_vett =@(x) ((Tw_req_k_vett./(x(2).*i_0tr*eta_pt)).*(((30*x(2)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)))./...
                     (((Tw_req_k_vett./(x(2).*i_0tr*eta_pt)).*(((30*x(2)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)))+...
                     (parte_1.*(((kc*(((Tw_req_k_vett./(T_em*x(2)))*T_em).^2).*(T_ref/T_em)^2))+...
                     (ki.*(((30*x(2)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).*(n_ref/n_mot_rad_s))+...
                     (kw.*((((30*x(2)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).^3).*(n_ref/n_mot_rad_s)^3)+C))); 

                 
                 
fun_eta_1_meno_2_vett =@(x) (((Tw_req_k_vett./(x(1).*i_0tr*eta_pt)).*(((30*x(1)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)))./...
                            (((Tw_req_k_vett./(x(1).*i_0tr*eta_pt)).*(((30*x(1)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)))+...
                            (parte_1.*(((kc*(((Tw_req_k_vett./(T_em*x(1)))*T_em).^2).*(T_ref/T_em)^2))+...
                            (ki.*(((30*x(1)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).*(n_ref/n_mot_rad_s))+...
                            (kw.*((((30*x(1)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).^3).*(n_ref/n_mot_rad_s)^3)+C))))-...
                            (((Tw_req_k_vett./(x(2).*i_0tr*eta_pt)).*(((30*x(2)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)))./...
                            (((Tw_req_k_vett./(x(2).*i_0tr*eta_pt)).*(((30*x(2)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)))+...
                            (parte_1.*(((kc*(((Tw_req_k_vett./(T_em*x(2)))*T_em).^2).*(T_ref/T_em)^2))+...
                            (ki.*(((30*x(2)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).*(n_ref/n_mot_rad_s))+...
                            (kw.*((((30*x(2)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).^3).*(n_ref/n_mot_rad_s)^3)+C))));

scelta_marcia_vett =@(x) (fun_eta_1_meno_2_vett(x)>=0)*x(1) + (fun_eta_1_meno_2_vett(x)<0)*x(2); 



P_gen_vett =@(x) (Tw_req_k_vett./(scelta_marcia_vett(x).*i_0tr*eta_pt)).*(((30*scelta_marcia_vett(x).*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30));
 
P_loss_vett =@(x)  parte_1.*(((kc*(((Tw_req_k_vett./(T_em*scelta_marcia_vett(x)))*T_em).^2).*(T_ref/T_em)^2))+...
                   (ki.*(((30*scelta_marcia_vett(x)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).*(n_ref/n_mot_rad_s))+...
                   (kw.*((((30*scelta_marcia_vett(x)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).^3).*(n_ref/n_mot_rad_s)^3)+C);

fun_eta_vett =@(x) ((Tw_req_k_vett./(scelta_marcia_vett(x).*i_0tr*eta_pt)).*(((30*scelta_marcia_vett(x)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)))./...
                   (((Tw_req_k_vett./(scelta_marcia_vett(x).*i_0tr*eta_pt)).*(((30*scelta_marcia_vett(x)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)))+...
                   (parte_1.*(((kc*(((Tw_req_k_vett./(T_em*scelta_marcia_vett(x)))*T_em).^2).*(T_ref/T_em)^2))+...
                   (ki.*(((30*scelta_marcia_vett(x)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).*(n_ref/n_mot_rad_s))+...
                   (kw.*((((30*scelta_marcia_vett(x)*i_0tr.*(vel_istanti(1:end-1)./3.6))./(pi*rt))*(pi/30)).^3).*(n_ref/n_mot_rad_s)^3)+C)));

               
              
fun_P_m_k_vett_1 =@(x) (F_accelerazione >=  0).*((fun_eta_vett(x)~= 0).*(((Tw_req_k_vett./(scelta_marcia_vett(x).*i_0tr*eta_pt)).*...
                                                                        ((30*i_0tr*scelta_marcia_vett(x).*(vel_istanti(1,2:end)/3.6))/(pi*rt)))./(9550*((fun_eta_vett(x) == 0)+fun_eta_vett(x))))...
                                               +(fun_eta_vett(x)== 0).*((parte_1*C)/1000));

fun_P_m_k_vett_2 =@(x) (F_accelerazione < 0).*((fun_eta_vett(x)~= 0).*((((Tw_req_k_vett_pm./(scelta_marcia_vett(x).*i_0tr*eta_pt)).*...
                                                                      ((30*i_0tr*scelta_marcia_vett(x).*(vel_istanti(1,2:end)/3.6))/(pi*rt)))./(9550)).*fun_eta_vett(x))...
                                               +(fun_eta_vett(x)== 0).*((parte_1*C)/1000));
                                           
fun_P_m_k_vett =@(x) fun_P_m_k_vett_1(x) +fun_P_m_k_vett_2(x); 


% Id_k =P_m_k*1000/Ud_k; %ampere
fun_Id_k_vett =@(x) fun_P_m_k_vett(x).*1000/Ud_k; %ampere

% delta_SOC =(Id_k/(3600*C_batt))*delta_t;
fun_SOC_spesa_vett =@(x) ((fun_P_m_k_vett(x).*1000/Ud_k)./(3600*C_batt))*delta_t;

% SOC =SOC-delta_SOC;  
fun_obiettivo =@(x) sum(fun_SOC_spesa_vett(x));



x0 =[i_1.min, i_2.min];
A =[];
B =[];
Aeq =[];
Beq =[];
lb =[i_1.min,i_2.min];
ub =[i_1.max,i_2.max];

nonlincon =@nlcon;

marce_SOC_migliore =fmincon(fun_obiettivo,x0,A,B,Aeq,Beq,lb,ub,nonlincon);
SOC_residuo = SOC-fun_obiettivo([marce_SOC_migliore(1),marce_SOC_migliore(2)]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parte v_0_100

val_iniziale=0;
val_finale =100/3.6;

% fun_intersez_1_inc =@(z,x) ((((z./((((2*pi)/60)*3.6).*(rt./x(1)))).*(pi/30)).*(Tw_req_50./(eta_pt.*x(1).*i_0tr)))./...
%                            ((((z./((((2*pi)/60)*3.6).*(rt./x(1)))).*(pi/30)).*(Tw_req_50./(eta_pt.*x(1).*i_0tr)))+...
%                            (parte_1.*((kc.*((Tw_req_50./(eta_pt.*x(1).*i_0tr)).^2).*(T_ref/T_em)^2)+...
%                            (ki.*((z./((((2*pi)/60)*3.6).*(rt./x(1)))).*(pi/30)).*(n_ref/n_mot_rad_s))+...
%                            (kw.*(((z./((((2*pi)/60)*3.6).*(rt./x(1)))).*(pi/30)).^3).*(n_ref/n_mot_rad_s)^3)+C)))) -...
%                            ((((z./((((2*pi)/60)*3.6).*(rt./x(2)))).*(pi/30)).*(Tw_req_50./(eta_pt.*x(2).*i_0tr)))./...
%                            ((((z./((((2*pi)/60)*3.6).*(rt./x(2)))).*(pi/30)).*(Tw_req_50./(eta_pt.*x(2).*i_0tr)))+...
%                            (parte_1.*((kc.*((Tw_req_50./(eta_pt.*x(2).*i_0tr)).^2).*(T_ref/T_em)^2)+...
%                            (ki.*((z./((((2*pi)/60)*3.6).*(rt./x(2)))).*(pi/30)).*(n_ref/n_mot_rad_s))+...
%                            (kw.*(((z./((((2*pi)/60)*3.6).*(rt./x(2)))).*(pi/30)).^3).*(n_ref/n_mot_rad_s)^3)+C))));

% fun_int_1 = @(z,x) (m_v*delta)./((((T_em*x(1))*v_climb*3.6)./(rt*3.6.*z)) - F_attrito_50 - (((Cd*S_ev)/(21.15))*((3.6.*z).^(2))));
fun_1 = @(z,x) (m_v*delta)./(  ((T_em*x(1)*eta_pt)./rt) - F_attrito_50 - ( ((Cd*S_ev)/(21.15))*(z.^(2)) )  );

fun_1_2 = @(z,x) (m_v*delta)./(  ((T_em*x(1).*(n_trans.*(((2*pi)/60)*3.6.*(rt./x(1))))/3.6)./(rt.*z)) - F_attrito_50 - ( ((Cd*S_ev)/(21.15))*(z.^(2)) )  );

% fun_int_2 = @(z,x) (m_v*delta)./(((T_em_maxspeed*x(2)*v_max*3.6)./(rt*3.6.*z)) - F_attrito_50 - (((Cd*S_ev)/(21.15))*((3.6.*z).^(2))));
fun_2 = @(z,x) (m_v*delta)./(  ((T_em*x(2).*(n_trans.*(((2*pi)/60)*3.6.*(rt./x(2))))/3.6)./(rt.*z)) - F_attrito_50 - ( ((Cd*S_ev)/(21.15))*(z.^(2)) )  );
          
            
int_1_funz =@(x) integral(@(z) fun_1(z,x),0,((n_trans.*(((2*pi)/60)*3.6*(rt./x(1))))./3.6));  
int_1_2_funz =@(x) integral(@(z) fun_1_2(z,x),((n_trans.*(((2*pi)/60)*3.6*(rt./x(1))))./3.6),(vel_cambio(x(1),x(2),P_nom,T_em,n_mot,rt))/3.6);
int_2_funz =@(x) integral(@(z) fun_2(z,x),(vel_cambio(x(1),x(2),P_nom,T_em,n_mot,rt))/3.6,val_finale);
int_cambio_funz =@(x) (0.5.*((vel_cambio(x(1),x(2),P_nom,T_em,n_mot,rt)) ~= 100));
            
int_finale =@(x) int_1_funz(x)+int_1_2_funz(x)+int_2_funz(x)+int_cambio_funz(x);
            
            
x0 =[i_1.min, i_2.min];
A =[];
B =[];
Aeq =[];
Beq =[];
lb =[i_1.min,i_2.min];
ub =[i_1.max,i_2.max];

nonlincon =@nlcon;

marce_0_100_migliore =fmincon(int_finale,x0,A,B,Aeq,Beq,lb,ub,nonlincon);

Vel_marce_scelte =int_finale([marce_0_100_migliore(1),marce_0_100_migliore(2)]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%mi trovo il valore max del SOC
fun_obiettivo_max =@(x) SOC-sum(fun_SOC_spesa_vett(x));

marce_SOC_migliore_max =fmincon(fun_obiettivo_max,x0,A,B,Aeq,Beq,lb,ub,nonlincon);
SOC_residuo_max = fun_obiettivo_max([marce_SOC_migliore_max(1),marce_SOC_migliore_max(2)]);


%mi trovo il valore max del metodo 0_100_km
int_finale_max=@(x) -(int_1_funz(x)+int_1_2_funz(x)+int_2_funz(x)+int_cambio_funz(x)); 
            
marce_0_100_migliore_max =fmincon(int_finale_max,x0,A,B,Aeq,Beq,lb,ub,nonlincon);
Vel_marce_scelte_max =-(int_finale_max([marce_0_100_migliore_max(1),marce_0_100_migliore_max(2)]));            


fprintf('\nil valore SOC residuo min è pari a: %.4f\n',SOC_residuo_max) 
fprintf('il valore della velocità max per portare il veicolo da 0-100 km/h è pari a: %.4f\n',Vel_marce_scelte_max)

fprintf('\nla combinazione di ingranaggi che ottimizza il valore SOC è:\nmarcia-1: %.2f\nmarcia-2: %.2f\n',marce_SOC_migliore(1),marce_SOC_migliore(2)) 
fprintf('\nil valore SOC residuo max è pari a: %.4f\n',SOC_residuo) 

fprintf('\nla combinazione di ingranaggi che mi ottimizza il tempo da 0-100 è:\nmarcia-1: %.2f\nmarcia-2: %.2f\n',marce_0_100_migliore(1),marce_0_100_migliore(2)) 
fprintf('\nil tempo minimo impegato per portare il veicolo da 0-100 km/h è pari a: %.2f secondi\n',Vel_marce_scelte) 


%mi calcolo le funzioni obiettivo da ottimizzare
peso_0_100 =1-peso_SOC;

funz_normaliz_SOC =@(x) (fun_obiettivo_max(x)-SOC_residuo_max)./(SOC_residuo-SOC_residuo_max);
funz_normaliz_0_100 =@(x) (Vel_marce_scelte_max-int_finale(x))./(Vel_marce_scelte_max-Vel_marce_scelte);

funz_globale_obiettivo =@(x) -(peso_SOC.*funz_normaliz_SOC(x) +peso_0_100.*funz_normaliz_0_100(x));

marce_globali_migliori =fmincon(funz_globale_obiettivo,x0,A,B,Aeq,Beq,lb,ub,nonlincon);
valore_marce_globali =-(funz_globale_obiettivo([marce_globali_migliori(1),marce_globali_migliori(2)]));

fprintf('\nil valore ponderato della coppia migliore di ingranggi è: %.4f\n',valore_marce_globali) 
fprintf('\nla combinazione di ingranaggi che ottimizza Globalmente è:\nmarcia-1: %.2f\nmarcia-2: %.2f\n',marce_globali_migliori(1),marce_globali_migliori(2)) 


